/*
 * Copyright (C) 2019-2021 Ashar Khan <ashar786khan@gmail.com>
 *
 * This file is part of CP Editor.
 *
 * CP Editor is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * I will not be responsible if CP Editor behaves in unexpected way and
 * causes your ratings to go down and or lose any important contest.
 *
 * Believe Software is "Software" and it isn't immune to bugs.
 *
 */

/*
 * This file is auto generated by src/Settings/genSettings.py
 */

#include "Settings/SettingsInfo.hpp"
#include "Editor/CodeEditor.hpp"
#include "Editor/KSHRepository.hpp"
#include "Core/StyleManager.hpp"
#include "Core/Translator.hpp"
#include "Settings/PathItem.hpp"
#include "Settings/ValueWrapper.hpp"
#include <QFontDatabase>
#include <QMessageBox>
#include <QRect>

QList<SettingsInfo::SettingInfo> SettingsInfo::settings;

void SettingsInfo::updateSettingInfo()
{
    settings.clear();
    settings.append(SettingInfo {"Tab Width", tr("Tab Width"), "Tab Width", "int", "", tr("The width of the tab character, or the number of spaces of an indent"), "The width of the tab character, or the number of spaces of an indent", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 4, QVariantList {1,16}});
    settings.append(SettingInfo {"Cursor Width", tr("Cursor Width"), "Cursor Width", "int", "", tr("The width of the cursor in pixels"), "The width of the cursor in pixels", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 1, QVariantList {1,10}});
    settings.append(SettingInfo {"Geometry", "Geometry", "Geometry", "QRect", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QRect(), QVariant()});
    settings.append(SettingInfo {"Editor Font", tr("Editor Font"), "Editor Font", "QFont", "", tr("The font of the code editor"), "The font of the code editor", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, [](){ QFont font = QFontDatabase::systemFont(QFontDatabase::FixedFont); font.setStyleHint(QFont::TypeWriter); return font; }(), true});
    settings.append(SettingInfo {"Use Custom Application Font", tr("Use Custom Application Font"), "Use Custom Application Font", "bool", "", tr("Use a custom font for the whole application instead of the default system font."), "Use a custom font for the whole application instead of the default system font.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Custom Application Font", tr("Custom Application Font"), "Custom Application Font", "QFont", "", tr("The custom font for the whole application"), "The custom font for the whole application", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Use Custom Application Font", [](const QVariant &var) { return var.toBool(); }}, }, {}, QFontDatabase::systemFont(QFontDatabase::GeneralFont), false});
    settings.append(SettingInfo {"Default Language", tr("Default Language"), "Default Language", "QString", "QComboBox", tr("The default language used when opening new tabs"), "The default language used when opening new tabs", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "C++", QStringList {"C++", "Java", "Python"}});
    settings.append(SettingInfo {"Clang Format/Program", tr("Clang Format Program"), "Clang Format Program", "QString", "PathItem", tr("The path to the Clang Format executable file"), "The path to the Clang Format executable file", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "clang-format", PathItem::Executable});
    settings.append(SettingInfo {"Clang Format/Arguments", tr("Clang Format Arguments"), "Clang Format Arguments", "QString", "", tr("The arguments passed to clang-format. It should NOT contain \"-i\"."), "The arguments passed to clang-format. It should NOT contain \"-i\".", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Clang Format/Style", tr("Clang Format Style"), "Clang Format Style", "QString", "QPlainTextEdit", tr("The Clang Format style options, which are usually saved in a .clang-format configuration file.\nYou can learn about it at <https://clang.llvm.org/docs/ClangFormatStyleOptions.html>."), "The Clang Format style options, which are usually saved in a .clang-format configuration file.\nYou can learn about it at <https://clang.llvm.org/docs/ClangFormatStyleOptions.html>.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "BasedOnStyle: Google", QVariant()});
    settings.append(SettingInfo {"YAPF/Program", tr("YAPF Program"), "YAPF Program", "QString", "PathItem", tr("The program of YAPF. It could be `yapf` (which doesn't need arguments) or `python` (which needs `-m yapf` as the arguments)."), "The program of YAPF. It could be `yapf` (which doesn't need arguments) or `python` (which needs `-m yapf` as the arguments).", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "python", PathItem::Executable});
    settings.append(SettingInfo {"YAPF/Arguments", tr("YAPF Arguments"), "YAPF Arguments", "QString", "", tr("The arguments passed to the YAPF program. It should NOT contain \"-i\"."), "The arguments passed to the YAPF program. It should NOT contain \"-i\".", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "-m yapf", QVariant()});
    settings.append(SettingInfo {"YAPF/Style", tr("YAPF Style"), "YAPF Style", "QString", "QPlainTextEdit", tr("The YAPF style options, which are usually saved in a .style.yapf or setup.conf configuration file.\nYou can learn about it by running `yapf --style-help`."), "The YAPF style options, which are usually saved in a .style.yapf or setup.conf configuration file.\nYou can learn about it by running `yapf --style-help`.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "[style]\nbased_on_style = pep8", QVariant()});
    settings.append(SettingInfo {"Format On Manual Save", tr("Format code on manual save"), "Format code on manual save", "bool", "", tr("Format the code when saving it manually."), "Format the code when saving it manually.", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Format On Auto Save", tr("Format code on auto-save"), "Format code on auto-save", "bool", "", tr("Format the code when auto-saving it."), "Format the code when auto-saving it.", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"C++/Template Path", tr("%1 Template Path").arg("C++"), "C++ Template Path", "QString", "PathItem", tr("The template used when creating a new C++ file"), "The template used when creating a new C++ file", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", PathItem::CppSource});
    settings.append(SettingInfo {"C++/Template Cursor Position Regex", tr("%1 Template Cursor Position Regex").arg("C++"), "C++ Template Cursor Position Regex", "QString", "", tr("The regular expression which matches a part of the code template.\nWhen opening a template, the position of the cursor is the position of the regex with an offset.\nThe cursor will be at the end of the template if there's no match of the regex."), "The regular expression which matches a part of the code template.\nWhen opening a template, the position of the cursor is the position of the regex with an offset.\nThe cursor will be at the end of the template if there's no match of the regex.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"C++/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, "main\\(\\)", QVariant()});
    settings.append(SettingInfo {"C++/Template Cursor Position Offset Type", tr("%1 Template Cursor Position Offset Type").arg("C++"), "C++ Template Cursor Position Offset Type", "QString", "QComboBox", tr("Whether the offset is relative to the start of the regex or the end of the regex."), "Whether the offset is relative to the start of the regex or the end of the regex.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"C++/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, "end", QStringList {"start", "end"}});
    settings.append(SettingInfo {"C++/Template Cursor Position Offset Characters", tr("%1 Template Cursor Position Offset Characters").arg("C++"), "C++ Template Cursor Position Offset Characters", "int", "", tr("The offset relative to the match of the regex in the number of characters, including white spaces."), "The offset relative to the match of the regex in the number of characters, including white spaces.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"C++/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, 4, QVariantList {-1000000, 1000000}});
    settings.append(SettingInfo {"C++/Compile Command", tr("%1 Compile Command").arg("C++"), "C++ Compile Command", "QString", "", tr("The command used to compile C++. It should NOT include the path to the source file or \"-o <output file>\"."), "The command used to compile C++. It should NOT include the path to the source file or \"-o <output file>\".", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "c++ -Wall", QVariant()});
    settings.append(SettingInfo {"C++/Output Path", tr("%1 Executable File Path").arg("C++"), "C++ Executable File Path", "QString", "", tr("The path of the compiled executable file.\nIt's relative to the source file, or the temporary directory if the tab is untitled.\nNo \".exe\" is needed.\nYou can use \"${filename}\" for the complete file name,\n\"${basename}\" for the base file name without the suffix,\n\"${tmpdir}\" or \"${tempdir}\" for the absolute path of the temporary directory."), "The path of the compiled executable file.\nIt's relative to the source file, or the temporary directory if the tab is untitled.\nNo \".exe\" is needed.\nYou can use \"${filename}\" for the complete file name,\n\"${basename}\" for the base file name without the suffix,\n\"${tmpdir}\" or \"${tempdir}\" for the absolute path of the temporary directory.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${tmpdir}/${basename}", QVariant()});
    settings.append(SettingInfo {"C++/Run Arguments", tr("%1 Run Arguments").arg("C++"), "C++ Run Arguments", "QString", "", tr("The runtime arguments when executing a C++ program"), "The runtime arguments when executing a C++ program", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"C++/Parentheses", "C++ Parentheses", "C++ Parentheses", "QVariantList", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QVariantList { QVariantList {'(', ')', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'{', '}', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'[', ']', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'"', '"', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'\'', '\'', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked} }, QVariant()});
    settings.append(SettingInfo {"Java/Template Path", tr("%1 Template Path").arg("Java"), "Java Template Path", "QString", "PathItem", tr("The template used when creating a new Java file"), "The template used when creating a new Java file", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", PathItem::JavaSource});
    settings.append(SettingInfo {"Java/Compile Command", tr("%1 Compile Command").arg("Java"), "Java Compile Command", "QString", "", tr("The command used to compile Java.\nIt should NOT include the path to the source file or the path of the compiled class file."), "The command used to compile Java.\nIt should NOT include the path to the source file or the path of the compiled class file.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "javac", QVariant()});
    settings.append(SettingInfo {"Java/Run Arguments", tr("%1 Run Arguments").arg("Java"), "Java Run Arguments", "QString", "", tr("The runtime arguments when executing a Java program"), "The runtime arguments when executing a Java program", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Java/Run Command", tr("%1 Run Command").arg("Java"), "Java Run Command", "QString", "", tr("The command to start a Java program. It should NOT include \"-classpath <path> <class name>\"."), "The command to start a Java program. It should NOT include \"-classpath <path> <class name>\".", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "java", QVariant()});
    settings.append(SettingInfo {"Java/Class Name", tr("%1 Class Name").arg("Java"), "Java Class Name", "QString", "", tr("The name of the main class of your solution."), "The name of the main class of your solution.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "Main", QVariant()});
    settings.append(SettingInfo {"Java/Output Path", tr("%1 Class Path").arg("Java"), "Java Class Path", "QString", "", tr("The path of the parent directory of the compiled class file.\nIt's relative to the source file, or the temporary directory if the tab is untitled.\nYou can use \"${filename}\" for the complete file name,\n\"${basename}\" for the base file name without the suffix,\n\"${tmpdir}\" or \"${tempdir}\" for the absolute path of the temporary directory."), "The path of the parent directory of the compiled class file.\nIt's relative to the source file, or the temporary directory if the tab is untitled.\nYou can use \"${filename}\" for the complete file name,\n\"${basename}\" for the base file name without the suffix,\n\"${tmpdir}\" or \"${tempdir}\" for the absolute path of the temporary directory.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${tmpdir}", QVariant()});
    settings.append(SettingInfo {"Java/Template Cursor Position Regex", tr("%1 Template Cursor Position Regex").arg("Java"), "Java Template Cursor Position Regex", "QString", "", tr("The regular expression which matches a part of the code template.\nWhen opening a template, the position of the cursor is the position of the regex with an offset.\nThe cursor will be at the end of the template if there's no match of the regex."), "The regular expression which matches a part of the code template.\nWhen opening a template, the position of the cursor is the position of the regex with an offset.\nThe cursor will be at the end of the template if there's no match of the regex.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Java/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, "", QVariant()});
    settings.append(SettingInfo {"Java/Template Cursor Position Offset Type", tr("%1 Template Cursor Position Offset Type").arg("Java"), "Java Template Cursor Position Offset Type", "QString", "QComboBox", tr("Whether the offset is relative to the start of the regex or the end of the regex."), "Whether the offset is relative to the start of the regex or the end of the regex.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Java/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, "end", QStringList {"start", "end"}});
    settings.append(SettingInfo {"Java/Template Cursor Position Offset Characters", tr("%1 Template Cursor Position Offset Characters").arg("Java"), "Java Template Cursor Position Offset Characters", "int", "", tr("The offset relative to the match of the regex in the number of characters, including white spaces."), "The offset relative to the match of the regex in the number of characters, including white spaces.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Java/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, 0, QVariantList {-1000000, 1000000}});
    settings.append(SettingInfo {"Java/Parentheses", "Java Parentheses", "Java Parentheses", "QVariantList", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QVariantList { QVariantList {'(', ')', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'{', '}', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'[', ']', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'"', '"', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'\'', '\'', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked} }, QVariant()});
    settings.append(SettingInfo {"Python/Template Path", tr("%1 Template Path").arg("Python"), "Python Template Path", "QString", "PathItem", tr("The template used when creating a new Python file"), "The template used when creating a new Python file", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", PathItem::PythonSource});
    settings.append(SettingInfo {"Python/Template Cursor Position Regex", tr("%1 Template Cursor Position Regex").arg("Python"), "Python Template Cursor Position Regex", "QString", "", tr("The regular expression which matches a part of the code template.\nWhen opening a template, the position of the cursor is the position of the regex with an offset.\nThe cursor will be at the end of the template if there's no match of the regex."), "The regular expression which matches a part of the code template.\nWhen opening a template, the position of the cursor is the position of the regex with an offset.\nThe cursor will be at the end of the template if there's no match of the regex.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Python/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, "", QVariant()});
    settings.append(SettingInfo {"Python/Template Cursor Position Offset Type", tr("%1 Template Cursor Position Offset Type").arg("Python"), "Python Template Cursor Position Offset Type", "QString", "QComboBox", tr("Whether the offset is relative to the start of the regex or the end of the regex."), "Whether the offset is relative to the start of the regex or the end of the regex.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Python/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, "end", QStringList {"start", "end"}});
    settings.append(SettingInfo {"Python/Template Cursor Position Offset Characters", tr("%1 Template Cursor Position Offset Characters").arg("Python"), "Python Template Cursor Position Offset Characters", "int", "", tr("The offset relative to the match of the regex in the number of characters, including white spaces."), "The offset relative to the match of the regex in the number of characters, including white spaces.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Python/Template Path", [](const QVariant &var) { return !var.toString().isEmpty(); }}, }, {}, 0, QVariantList {-1000000, 1000000}});
    settings.append(SettingInfo {"Python/Run Arguments", tr("%1 Run Arguments").arg("Python"), "Python Run Arguments", "QString", "", tr("The runtime arguments when executing a Python program"), "The runtime arguments when executing a Python program", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Python/Run Command", tr("%1 Run Command").arg("Python"), "Python Run Command", "QString", "", tr("The command to start a Python program. It should NOT include the path to the source file."), "The command to start a Python program. It should NOT include the path to the source file.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "python", QVariant()});
    settings.append(SettingInfo {"Python/Parentheses", "Python Parentheses", "Python Parentheses", "QVariantList", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QVariantList { QVariantList {'(', ')', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'{', '}', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'[', ']', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'"', '"', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked}, QVariantList {'\'', '\'', Qt::PartiallyChecked, Qt::PartiallyChecked, Qt::PartiallyChecked} }, QVariant()});
    settings.append(SettingInfo {"Editor Theme", tr("Editor Theme"), "Editor Theme", "QString", "QComboBox", tr("The syntax highlight theme of the code editor"), "The syntax highlight theme of the code editor", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "Default", Editor::KSyntaxHighlightingRepository::themeNames()});
    settings.append(SettingInfo {"Highlight Error Line", tr("Highlight Error Line"), "Highlight Error Line", "bool", "", tr("Highlight lines containing diagnostics"), "Highlight lines containing diagnostics", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Detached Run Terminal Program", tr("Terminal Program"), "Terminal Program", "QString", "", tr("The terminal emulator to use when running in detached mode.\nThis is usually the name or the path of the terminal emulator.\nSome possible values are konsole, gnome-terminal, xfce-terminal, xterm or any other terminal emulator program."), "The terminal emulator to use when running in detached mode.\nThis is usually the name or the path of the terminal emulator.\nSome possible values are konsole, gnome-terminal, xfce-terminal, xterm or any other terminal emulator program.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "xterm", QVariant()});
    settings.append(SettingInfo {"Detached Run Terminal Arguments", tr("Terminal Arguments"), "Terminal Arguments", "QString", "", tr("Arguments used to execute a given command in the terminal emulator.\nThis is \"-e\" for most terminal emulators, including konsole, xterm, xfce-terminal but can be \"--\" for gnome-terminal.\nConsult your terminal emulator for the suitable arguments."), "Arguments used to execute a given command in the terminal emulator.\nThis is \"-e\" for most terminal emulators, including konsole, xterm, xfce-terminal but can be \"--\" for gnome-terminal.\nConsult your terminal emulator for the suitable arguments.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "-e", QVariant()});
    settings.append(SettingInfo {"Auto Complete Parentheses", tr("Auto Complete Parentheses"), "Auto Complete Parentheses", "bool", "", tr("Automatically complete a pair of parentheses when typing the left element of it,\nand move out of it when typing the right element of it.\nThis can be overridden for each parenthesis in each language."), "Automatically complete a pair of parentheses when typing the left element of it,\nand move out of it when typing the right element of it.\nThis can be overridden for each parenthesis in each language.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Auto Remove Parentheses", tr("Auto Remove Parentheses"), "Auto Remove Parentheses", "bool", "", tr("Automatically delete the whole pair of parentheses when deleting\nthe left element of it if the two elements are adjacent.\nThis can be overridden for each parenthesis in each language."), "Automatically delete the whole pair of parentheses when deleting\nthe left element of it if the two elements are adjacent.\nThis can be overridden for each parenthesis in each language.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Tab Jump Out Parentheses", tr("Jump out of a parenthesis by pressing Tab"), "Jump out of a parenthesis by pressing Tab", "bool", "", tr("When this is enabled, you can use Tab instead of the\nclosing parenthesis to jump out of a parenthesis.\nThis can be overridden for each parenthesis in each language."), "When this is enabled, you can use Tab instead of the\nclosing parenthesis to jump out of a parenthesis.\nThis can be overridden for each parenthesis in each language.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Auto Indent", tr("Auto Indent"), "Auto Indent", "bool", "", tr("Add an indent when entering a new line after a \"{\"."), "Add an indent when entering a new line after a \"{\".", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Auto Save", tr("Enable Auto Save"), "Enable Auto Save", "bool", "", tr(""), "", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Auto Save Interval", tr("Auto Save Interval (ms)"), "Auto Save Interval (ms)", "int", "", tr("The time interval between a modification and an auto-save, or between two auto-saves."), "The time interval between a modification and an auto-save, or between two auto-saves.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Auto Save", [](const QVariant &var) { return var.toBool(); }}, }, {}, 1000, QVariantList {100, 36000000, 1000}});
    settings.append(SettingInfo {"Auto Save Interval Type", tr("Auto Save Interval Type"), "Auto Save Interval Type", "QString", "QComboBox", tr("After the last modification: the timer will be reset after a modification to the code.\nAfter the first modification: the timer will start after a modification if at that time the timer is not running.\nWithout modification: auto-save happens with a constant interval no matter there are modifications or not."), "After the last modification: the timer will be reset after a modification to the code.\nAfter the first modification: the timer will start after a modification if at that time the timer is not running.\nWithout modification: auto-save happens with a constant interval no matter there are modifications or not.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Auto Save", [](const QVariant &var) { return var.toBool(); }}, }, {}, "After the last modification", QStringList { "After the last modification", "After the first modification", "Without modification" }});
    settings.append(SettingInfo {"Wrap Text", tr("Wrap Text"), "Wrap Text", "bool", "", tr("Wrap a line into several lines if it doesn't fit into the screen."), "Wrap a line into several lines if it doesn't fit into the screen.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Beta", tr("Use the beta version"), "Use the beta version", "bool", "", tr("Check for updates marked as pre-releases, which are considered not very stable but have more features."), "Check for updates marked as pre-releases, which are considered not very stable but have more features.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Replace Tabs", tr("Replace tabs by spaces"), "Replace tabs by spaces", "bool", "", tr("Use spaces instead of a tab character."), "Use spaces instead of a tab character.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Save Tests", tr("Save Testcases on Save"), "Save Testcases on Save", "bool", "", tr("Save the test cases on the disk when saving a file, and load the saved test cases when opening a file."), "Save the test cases on the disk when saving a file, and load the saved test cases when opening a file.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Maximized Window", "Maximized Window", "Maximized Window", "bool", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Check Update", tr("Check for updates on startup"), "Check for updates on startup", "bool", "", tr("Check whether there's a new version of CP Editor when starting CP Editor."), "Check whether there's a new version of CP Editor when starting CP Editor.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Opacity", tr("Opacity"), "Opacity", "int", "QSlider", tr("The opacity of the main window"), "The opacity of the main window", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 100, QVariantList {60,100}});
    settings.append(SettingInfo {"View Mode", "View Mode", "View Mode", "QString", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "split", QVariant()});
    settings.append(SettingInfo {"Splitter Size", "Splitter Size", "Splitter Size", "QByteArray", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QByteArray(), QVariant()});
    settings.append(SettingInfo {"Right Splitter Size", "Right Splitter Size", "Right Splitter Size", "QByteArray", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QByteArray(), QVariant()});
    settings.append(SettingInfo {"Competitive Companion/Enable", tr("Enable Competitive Companion"), "Enable Competitive Companion", "bool", "", tr("Receive data sent by Competitive Companion and load the example test cases."), "Receive data sent by Competitive Companion and load the example test cases.", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Competitive Companion/Connection Port", tr("Connection Port"), "Connection Port", "int", "", tr("The port used to receive data from Competitive Companion"), "The port used to receive data from Competitive Companion", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Competitive Companion/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, 10045, QVariantList {1024,49151}});
    settings.append(SettingInfo {"Competitive Companion/Open New Tab", tr("Open New Tabs"), "Open New Tabs", "bool", "", tr("Open a new tab for each problem parsed by Competitive Companion."), "Open a new tab for each problem parsed by Competitive Companion.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Competitive Companion/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, true, QVariant()});
    settings.append(SettingInfo {"Competitive Companion/Set Time Limit For Tab", tr("Use the time limit from Competitive Companion"), "Use the time limit from Competitive Companion", "bool", "", tr("Use the time limit parsed by Competitive Companion as the time limit of the corresponding tab."), "Use the time limit parsed by Competitive Companion as the time limit of the corresponding tab.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Competitive Companion/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, false, QVariant()});
    settings.append(SettingInfo {"Competitive Companion/Head Comments", tr("Content of the head comments"), "Content of the head comments", "QString", "QPlainTextEdit", tr("The comments added at the head of the code when parsing a problem.\nAvailable place holders are:\n${time}: the time when parsing the problem\n${json.X.Y}: an attribute of the data provided by Competitive Companion, you can read more at https://github.com/jmerle/competitive-companion#explanation"), "The comments added at the head of the code when parsing a problem.\nAvailable place holders are:\n${time}: the time when parsing the problem\n${json.X.Y}: an attribute of the data provided by Competitive Companion, you can read more at https://github.com/jmerle/competitive-companion#explanation", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Competitive Companion/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, "Problem: ${json.name}\nContest: ${json.group}\nURL: ${json.url}\nMemory Limit: ${json.memoryLimit} MB\nTime Limit: ${json.timeLimit} ms", QVariant()});
    settings.append(SettingInfo {"Competitive Companion/Head Comments Time Format", tr("Time format for the head comments"), "Time format for the head comments", "QString", "", tr("The format of the ${time} place holder in the head comments.\nYou can read the Qt documentation for available expressions:\nhttps://doc.qt.io/qt-5/qdate.html#toString\nhttps://doc.qt.io/qt-5/qtime.html#toString"), "The format of the ${time} place holder in the head comments.\nYou can read the Qt documentation for available expressions:\nhttps://doc.qt.io/qt-5/qdate.html#toString\nhttps://doc.qt.io/qt-5/qtime.html#toString", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Competitive Companion/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, "yyyy-MM-dd HH:mm:ss", QVariant()});
    settings.append(SettingInfo {"Competitive Companion/Head Comments Powered By CP Editor", tr("Add \"Powered By CP Editor\" in the head comments"), "Add \"Powered By CP Editor\" in the head comments", "bool", "", tr("Add a line saying \"Powered By CP Editor\" in the head comments.\nThis doesn't cost you anything, but helps more people to know CP Editor."), "Add a line saying \"Powered By CP Editor\" in the head comments.\nThis doesn't cost you anything, but helps more people to know CP Editor.", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Competitive Companion/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, true, QVariant()});
    settings.append(SettingInfo {"Hotkey/Format", tr("Format Codes"), "Format Codes", "QString", "ShortcutItem", tr(""), "", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Hotkey/Kill", tr("Kill All Processes"), "Kill All Processes", "QString", "ShortcutItem", tr(""), "", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Hotkey/Compile Run", tr("Compile and Run"), "Compile and Run", "QString", "ShortcutItem", tr(""), "", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Hotkey/Run", tr("Run Only"), "Run Only", "QString", "ShortcutItem", tr(""), "", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Hotkey/Compile", tr("Compile Only"), "Compile Only", "QString", "ShortcutItem", tr(""), "", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Hotkey/Change View Mode", tr("Change View Mode"), "Change View Mode", "QString", "ShortcutItem", tr(""), "", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Hotkey/Snippets", tr("Use Snippets"), "Use Snippets", "QString", "ShortcutItem", tr(""), "", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "", QVariant()});
    settings.append(SettingInfo {"Hot Exit/Enable", tr("Restore last session at startup"), "Restore last session at startup", "bool", "", tr("Restore the last session when the application starts.\nWhen this is enabled, you won't be asked whether to save unsaved files when exiting."), "Restore the last session when the application starts.\nWhen this is enabled, you won't be asked whether to save unsaved files when exiting.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Hot Exit/Auto Save", tr("Auto-save the current session periodically"), "Auto-save the current session periodically", "bool", "", tr("Auto-save the current session periodically instead of only save when the application exists.\nThis is useful if your computer is frozen and you have to cut off the power or\nkill the application with SIGKILL which could not be handled by the application."), "Auto-save the current session periodically instead of only save when the application exists.\nThis is useful if your computer is frozen and you have to cut off the power or\nkill the application with SIGKILL which could not be handled by the application.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Hot Exit/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, false, QVariant()});
    settings.append(SettingInfo {"Hot Exit/Auto Save Interval", tr("Auto-save Session Interval"), "Auto-save Session Interval", "int", "", tr("The time interval between two auto-saves of the current session."), "The time interval between two auto-saves of the current session.", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Hot Exit/Enable", [](const QVariant &var) { return var.toBool(); }}, {"Hot Exit/Auto Save", [](const QVariant &var) { return var.toBool(); }}, }, {}, 20000, QVariantList {1000, 36000000, 1000}});
    settings.append(SettingInfo {"Force Close", "Force Close", "Force Close", "bool", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"CF/Path", tr("Path"), "Path", "QString", "PathItem", tr("The path to the CF Tool executable file"), "The path to the CF Tool executable file", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "cf", PathItem::Executable});
    settings.append(SettingInfo {"CF/Show Toast Messages", tr("Show toast messages for submission verdicts"), "Show toast messages for submission verdicts", "bool", "", tr("Show a toast message when the verdict of a submission is known. You can see the message outside of CP Editor."), "Show a toast message when the verdict of a submission is known. You can see the message outside of CP Editor.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Show Compile And Run Only", tr("Show Compile And Run Only"), "Show Compile And Run Only", "bool", "", tr("Hide the Compile Only button and the Run Only button under the code editor in the main window."), "Hide the Compile Only button and the Run Only button under the code editor in the main window.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Display EOLN In Diff", tr("Display EOLN In Diff"), "Display EOLN In Diff", "bool", "", tr("Use \"\u00b6\" to represent for the new line character in the HTML Diff Viewer."), "Use \"\u00b6\" to represent for the new line character in the HTML Diff Viewer.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Save Faster", tr("Save Files Faster"), "Save Files Faster", "bool", "", tr("Always use QFile instead of QSaveFile to save files.\nThis will be faster but with a little bit more risk of losing the file (with a very small possibility)."), "Always use QFile instead of QSaveFile to save files.\nThis will be faster but with a little bit more risk of losing the file (with a very small possibility).", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Default Time Limit", tr("Default Time Limit (ms)"), "Default Time Limit (ms)", "int", "", tr("The default time limit when executing the program.\nThe program will be killed if it doesn't terminate in the time limit."), "The default time limit when executing the program.\nThe program will be killed if it doesn't terminate in the time limit.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 5000, QVariantList {1,3600000,1000}});
    settings.append(SettingInfo {"Output Length Limit", tr("Output Length Limit"), "Output Length Limit", "int", "", tr("The maximum number of characters in the output of the program.\nThe program will be killed if either of its stdout or stderr is too long."), "The maximum number of characters in the output of the program.\nThe program will be killed if either of its stdout or stderr is too long.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 500000, QVariantList {2,1000000000}});
    settings.append(SettingInfo {"Output Display Length Limit", tr("Output Display Length Limit"), "Output Display Length Limit", "int", "", tr("The maximum number of characters to be displayed for the output of the program.\nIf the output is too long, it will be elided."), "The maximum number of characters to be displayed for the output of the program.\nIf the output is too long, it will be elided.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 50000, QVariantList {2,100000000}});
    settings.append(SettingInfo {"Message Length Limit", tr("Message Length Limit"), "Message Length Limit", "int", "", tr("The maximum number of characters in each message in the top-right corner of the main window.\nThe message will be elided if it's too long."), "The maximum number of characters in each message in the top-right corner of the main window.\nThe message will be elided if it's too long.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 20000, QVariantList {500,100000000}});
    settings.append(SettingInfo {"HTML Diff Viewer Length Limit", tr("HTML Diff Viewer Length Limit"), "HTML Diff Viewer Length Limit", "int", "", tr("The maximum number of characters in the HTML Diff Viewer.\nThe Diff Viewer will fall back to plain text if either of the output or the expected output is too long."), "The maximum number of characters in the HTML Diff Viewer.\nThe Diff Viewer will fall back to plain text if either of the output or the expected output is too long.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 50000, QVariantList {2,100000000}});
    settings.append(SettingInfo {"Open File Length Limit", tr("Open File Length Limit"), "Open File Length Limit", "int", "", tr("The maximum number of characters in a source file to open.\nA source file won't be opened if it's too long."), "The maximum number of characters in a source file to open.\nA source file won't be opened if it's too long.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 20000, QVariantList {2,100000000}});
    settings.append(SettingInfo {"Display Test Case Length Limit", tr("Display Test Case Length Limit"), "Display Test Case Length Limit", "int", "", tr("The maximum number of characters in a test case to be displayed.\nA test case will be elided and read-only if it's too long."), "The maximum number of characters in a test case to be displayed.\nA test case will be elided and read-only if it's too long.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 500000, QVariantList {2,1000000000}});
    settings.append(SettingInfo {"LSP/Path C++", tr("Path to LSP executable"), "Path to LSP executable", "QString", "PathItem", tr("The path to the C++ Language Server executable"), "The path to the C++ Language Server executable", "", false, false, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting C++", [](const QVariant &var) { return var.toBool(); }}, }, {}, "clangd", PathItem::Executable});
    settings.append(SettingInfo {"LSP/Path Java", tr("Path to LSP executable"), "Path to LSP executable", "QString", "PathItem", tr("The path to the Java Language Server executable"), "The path to the Java Language Server executable", "", false, false, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting Java", [](const QVariant &var) { return var.toBool(); }}, }, {}, "", PathItem::Executable});
    settings.append(SettingInfo {"LSP/Path Python", tr("Path to LSP executable"), "Path to LSP executable", "QString", "PathItem", tr("The path to the Python Language Server executable"), "The path to the Python Language Server executable", "", false, false, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting Python", [](const QVariant &var) { return var.toBool(); }}, }, {}, "python", PathItem::Executable});
    settings.append(SettingInfo {"LSP/Use Linting C++", tr("Use Linting with Language Server"), "Use Linting with Language Server", "bool", "", tr("Show Error, Warning, Information and Hints in Code Editor for C++ Language"), "Show Error, Warning, Information and Hints in Code Editor for C++ Language", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"LSP/Use Linting Java", tr("Use Linting with Language Server"), "Use Linting with Language Server", "bool", "", tr("Show Error, Warning, Information and Hints in Code Editor for Java Language"), "Show Error, Warning, Information and Hints in Code Editor for Java Language", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"LSP/Use Linting Python", tr("Use Linting with Language Server"), "Use Linting with Language Server", "bool", "", tr("Show Error, Warning, Information and Hints in Code Editor for Python Language"), "Show Error, Warning, Information and Hints in Code Editor for Python Language", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"LSP/Use Autocomplete C++", tr("Use auto-complete with Language Server"), "Use auto-complete with Language Server", "bool", "", tr("Use autocomplete results from Language server"), "Use autocomplete results from Language server", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"LSP/Use Autocomplete Java", tr("Use auto-complete with Language Server"), "Use auto-complete with Language Server", "bool", "", tr("Use autocomplete results from Language server"), "Use autocomplete results from Language server", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"LSP/Use Autocomplete Python", tr("Use auto-complete with Language Server"), "Use auto-complete with Language Server", "bool", "", tr("Use autocomplete results from Language server"), "Use autocomplete results from Language server", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"LSP/Delay C++", tr("Delay in Linting (ms)"), "Delay in Linting (ms)", "int", "", tr("Delay in linting in milliseconds after last modification to code"), "Delay in linting in milliseconds after last modification to code", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting C++", [](const QVariant &var) { return var.toBool(); }}, }, {}, 2000, QVariantList {10, 3600000, 1000}});
    settings.append(SettingInfo {"LSP/Delay Java", tr("Delay in Linting (ms)"), "Delay in Linting (ms)", "int", "", tr("Delay in linting in milliseconds after last modification to code"), "Delay in linting in milliseconds after last modification to code", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting Java", [](const QVariant &var) { return var.toBool(); }}, }, {}, 2000, QVariantList {10, 3600000, 1000}});
    settings.append(SettingInfo {"LSP/Delay Python", tr("Delay in Linting (ms)"), "Delay in Linting (ms)", "int", "", tr("Delay in linting in milliseconds after last modification to code"), "Delay in linting in milliseconds after last modification to code", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting Python", [](const QVariant &var) { return var.toBool(); }}, }, {}, 2000, QVariantList {10, 3600000, 1000}});
    settings.append(SettingInfo {"LSP/Args C++", tr("Arguments for Language Server"), "Arguments for Language Server", "QString", "", tr("Arguments to pass to Language server executable"), "Arguments to pass to Language server executable", "", false, false, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting C++", [](const QVariant &var) { return var.toBool(); }}, }, {}, "", QVariant()});
    settings.append(SettingInfo {"LSP/Args Java", tr("Arguments for Language Server"), "Arguments for Language Server", "QString", "", tr("Arguments to pass to Language server executable"), "Arguments to pass to Language server executable", "", false, false, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting Java", [](const QVariant &var) { return var.toBool(); }}, }, {}, "", QVariant()});
    settings.append(SettingInfo {"LSP/Args Python", tr("Arguments for Language Server"), "Arguments for Language Server", "QString", "", tr("Arguments to pass to Language server executable"), "Arguments to pass to Language server executable", "", false, false, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"LSP/Use Linting Python", [](const QVariant &var) { return var.toBool(); }}, }, {}, "-m pyls", QVariant()});
    settings.append(SettingInfo {"Testcases Matching Rules", tr("Testcases Matching Rules"), "Testcases Matching Rules", "QVariantList", "", tr("Pairs of regular expressions used when adding pairs of test cases from files.\nEach pair of regular expressions represents a test case."), "Pairs of regular expressions used when adding pairs of test cases from files.\nEach pair of regular expressions represents a test case.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QVariantList { QStringList {"(.*)\\.in", "\\1.ans"}, QStringList {"(.*)\\.in", "\\1.out"}}, QVariantList { QStringList {tr("Input Regex"), tr("The regular expression which matches the whole input file name")}, QStringList {tr("Answer Replace"), tr("The replace expression for the answer file name.\nYou can use \"\\1\" for the first captured group.")}}});
    settings.append(SettingInfo {"Input File Save Path", tr("Input File Save Path"), "Input File Save Path", "QString", "", tr("The path where the input files are saved.\nThis setting is a relative path to the source file.\nYou can use \"${filename}\" for the complete file name,\n\"${basename}\" for the base file name without the suffix,\n\"${0-index}\" for the index of the test case started from 0,\n\"${1-index}\" for the index of the test case started from 1."), "The path where the input files are saved.\nThis setting is a relative path to the source file.\nYou can use \"${filename}\" for the complete file name,\n\"${basename}\" for the base file name without the suffix,\n\"${0-index}\" for the index of the test case started from 0,\n\"${1-index}\" for the index of the test case started from 1.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "./${basename}_${1-index}.in", QVariant()});
    settings.append(SettingInfo {"Answer File Save Path", tr("Answer File Save Path"), "Answer File Save Path", "QString", "", tr("The path where the answer files are saved.\nThis setting is a relative path to the source file.\nYou can use \"${filename}\" for the complete file name,\n\"${basename}\" for the base file name without the suffix,\n\"${0-index}\" for the index of the test case started from 0,\n\"${1-index}\" for the index of the test case started from 1."), "The path where the answer files are saved.\nThis setting is a relative path to the source file.\nYou can use \"${filename}\" for the complete file name,\n\"${basename}\" for the base file name without the suffix,\n\"${0-index}\" for the index of the test case started from 0,\n\"${1-index}\" for the index of the test case started from 1.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "./${basename}_${1-index}.ans", QVariant()});
    settings.append(SettingInfo {"Default File Paths For Problem URLs", tr("Default File Paths For Problem URLs"), "Default File Paths For Problem URLs", "QVariantList", "", tr("The default file path used when saving a new file while the problem URL is set"), "The default file path used when saving a new file while the problem URL is set", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QVariantList {}, QVariantList { QStringList { tr("Problem URL"), tr("The regular expression which matches a part of the problem URL")}, QStringList {tr("File Path"), tr("The replace expression for the file path, without file name suffix.\nYou can use \"\\1\" for the first captured group.")}}});
    settings.append(SettingInfo {"Test Cases Font", tr("Test Cases Font"), "Test Cases Font", "QFont", "", tr("The font of test cases"), "The font of test cases", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, [](){ QFont font = QFontDatabase::systemFont(QFontDatabase::FixedFont); font.setStyleHint(QFont::TypeWriter); return font; }(), true});
    settings.append(SettingInfo {"Extra Bottom Margin", tr("Add extra margin at the bottom of the code editor"), "Add extra margin at the bottom of the code editor", "bool", "", tr("Add an extra margin with the height of a page at the bottom of the code editor.\nDue to technical reasons, changing the height of the margin affects the undo history."), "Add an extra margin with the height of a page at the bottom of the code editor.\nDue to technical reasons, changing the height of the margin affects the undo history.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Message Logger Font", tr("Message Logger Font"), "Message Logger Font", "QFont", "", tr("The font of the message logger"), "The font of the message logger", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, [](){ QFont font = QFontDatabase::systemFont(QFontDatabase::FixedFont); font.setStyleHint(QFont::TypeWriter); return font; }(), true});
    settings.append(SettingInfo {"Save File On Compilation", tr("Save File On Compilation"), "Save File On Compilation", "bool", "", tr("Save the source file when compiling it. It won't be saved if the tab is untitled."), "Save the source file when compiling it. It won't be saved if the tab is untitled.", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Save File On Execution", tr("Save File On Execution"), "Save File On Execution", "bool", "", tr("Save the source file when running it. It won't be saved if the tab is untitled."), "Save the source file when running it. It won't be saved if the tab is untitled.", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Restore Old Problem Url", tr("Restore the problem URL when opening a file"), "Restore the problem URL when opening a file", "bool", "", tr("If a problem URL was set for a file, when you open\nthat file again, the problem URL will be restored."), "If a problem URL was set for a file, when you open\nthat file again, the problem URL will be restored.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Open Old File For Old Problem Url", tr("Open the old file when parsing an old problem URL"), "Open the old file when parsing an old problem URL", "bool", "", tr("If a problem URL was set for a file, when parsing that problem\nfrom Competitive Companion again, the old file will be opened."), "If a problem URL was set for a file, when parsing that problem\nfrom Competitive Companion again, the old file will be opened.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Locale", tr("UI Language"), "UI Language", "QString", "QComboBox", tr("The language displayed in the UI."), "The language displayed in the UI.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){ Core::Translator::setLocale(); QMessageBox::warning(parent, tr("Change Locale"), tr("You need to restart the application to completely apply the locale change.")); }, {}, {}, "system", QStringList { "system", "Νέα Ελληνικά", "English", "Español", "Português brasileiro", "Русский", "简体中文", "正體中文" }});
    settings.append(SettingInfo {"UI Style", tr("UI Style"), "UI Style", "QString", "QComboBox", tr("The style of the whole application."), "The style of the whole application.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "default", Core::StyleManager::styleList()});
    QList<SettingInfo> LISTLanguageConfig;
    LISTLanguageConfig.append(SettingInfo {"Snippet", "Snippet", "Snippet", "QMap:QString", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QVariantMap(), QVariant()});
    settings.append(SettingInfo {"Language Config", "Language Config", "Language Config", "Object", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QVariant(), QVariant(), LISTLanguageConfig});
    settings.append(SettingInfo {"Run On Empty Testcase", tr("Run your codes on empty test cases"), "Run your codes on empty test cases", "bool", "", tr("Run your code on all non-hidden test cases even if the input is empty."), "Run your code on all non-hidden test cases even if the input is empty.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Check On Testcases With Empty Output", tr("Check your answer on test cases with empty output"), "Check your answer on test cases with empty output", "bool", "", tr("Check your answer even if your output or the expected output is empty."), "Check your answer even if your output or the expected output is empty.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Test Case Maximum Height", tr("Test Case Maximum Height"), "Test Case Maximum Height", "int", "", tr("The maximum height of a test case without a scrollbar in pixels."), "The maximum height of a test case without a scrollbar in pixels.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 300, QVariantList {30, 2000}});
    settings.append(SettingInfo {"Proxy/Enabled", tr("Enable proxy for checking updates"), "Enable proxy for checking updates", "bool", "", tr("Enable proxy to connect to GitHub while checking updates"), "Enable proxy to connect to GitHub while checking updates", tr("network-proxy", "the anchor of Enable proxy for checking updates on the corresponding page of https://cpeditor.org/docs/preferences"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Proxy/Type", tr("Type"), "Type", "QString", "QComboBox", tr("The type of the proxy. \"System\" for using the system proxy."), "The type of the proxy. \"System\" for using the system proxy.", tr("network-proxy", "the anchor of Type on the corresponding page of https://cpeditor.org/docs/preferences"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "System", QStringList { "System", "Http", "Socks5" }});
    settings.append(SettingInfo {"Proxy/Host Name", tr("Host Name"), "Host Name", "QString", "", tr("The hostname of the proxy, e.g. 127.0.0.1"), "The hostname of the proxy, e.g. 127.0.0.1", tr("network-proxy", "the anchor of Host Name on the corresponding page of https://cpeditor.org/docs/preferences"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Proxy/Type", [](const QVariant &var) { return var.toString() != "System"; }}, }, {}, "127.0.0.1", QVariant()});
    settings.append(SettingInfo {"Proxy/Port", tr("Port"), "Port", "int", "", tr("The port of the proxy, e.g. 1080"), "The port of the proxy, e.g. 1080", tr("network-proxy", "the anchor of Port on the corresponding page of https://cpeditor.org/docs/preferences"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Proxy/Type", [](const QVariant &var) { return var.toString() != "System"; }}, }, {}, 1080, QVariantList {1024,49151}});
    settings.append(SettingInfo {"Proxy/User", tr("User"), "User", "QString", "", tr("The user of the proxy server. It can be empty if the proxy server doesn't require authentication."), "The user of the proxy server. It can be empty if the proxy server doesn't require authentication.", tr("network-proxy", "the anchor of User on the corresponding page of https://cpeditor.org/docs/preferences"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Proxy/Type", [](const QVariant &var) { return var.toString() != "System"; }}, }, {}, "", QVariant()});
    settings.append(SettingInfo {"Proxy/Password", tr("Password"), "Password", "QString", "", tr("The password of the proxy server. It can be empty if the proxy server doesn't require authentication."), "The password of the proxy server. It can be empty if the proxy server doesn't require authentication.", tr("network-proxy", "the anchor of Password on the corresponding page of https://cpeditor.org/docs/preferences"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Proxy/Type", [](const QVariant &var) { return var.toString() != "System"; }}, }, {}, "", QVariant()});
    settings.append(SettingInfo {"C++/Compiler Output Codec", tr("%1 Compiler Output Codec").arg("C++"), "C++ Compiler Output Codec", "QString", "CodecBox", tr("Text codec of the compiler output (errors, warnings, etc.)"), "Text codec of the compiler output (errors, warnings, etc.)", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "UTF-8", QVariant()});
    settings.append(SettingInfo {"Java/Compiler Output Codec", tr("%1 Compiler Output Codec").arg("Java"), "Java Compiler Output Codec", "QString", "CodecBox", tr("Text codec of the compiler output (errors, warnings, etc.)"), "Text codec of the compiler output (errors, warnings, etc.)", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "UTF-8", QVariant()});
    settings.append(SettingInfo {"Default Path/Names And Paths", tr("Default Path Names And Paths"), "Default Path Names And Paths", "QVariantList", "", tr("A list of default paths.\nThey can be used in actions' corresponding default paths by using ${<default path name>} as a place holder.\nThey can be either manually set or automatically changed after choosing a path for an action."), "A list of default paths.\nThey can be used in actions' corresponding default paths by using ${<default path name>} as a place holder.\nThey can be either manually set or automatically changed after choosing a path for an action.", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QVariantList(), QVariantList { QStringList { tr("Name"), tr("The name of a default path") }, QStringList { tr("Path"), tr("The path of a default path") } }});
    settings.append(SettingInfo {"Auto Load External Changes If No Unsaved Modification", tr("Auto-load external file changes if there's no unsaved modification"), "Auto-load external file changes if there's no unsaved modification", "bool", "", tr("Automatically load file changes that are not made in CP Editor if there's no unsaved modification in CP Editor."), "Automatically load file changes that are not made in CP Editor if there's no unsaved modification in CP Editor.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Ask For Loading External Changes", tr("Ask whether to load external file changes"), "Ask whether to load external file changes", "bool", "", tr("When there are file changes that are not made in CP Editor and is not automatically loaded by\n\"%1\", ask for whether to load the changes.\nIf this is disabled, external file changes will be ignored unless they are loaded by\n\"%1\".").arg(tr("Auto-load external file changes if there's no unsaved modification")), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Recent Files", "Recent Files", "Recent Files", "QStringList", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, QStringList(), QVariant()});
    settings.append(SettingInfo {"Number Of Problems In Contest", "Number Of Problems In Contest", "Number Of Problems In Contest", "int", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 5, QVariant()});
    settings.append(SettingInfo {"First Time User", "First Time User", "First Time User", "bool", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Show Only Monospaced Font", tr("Show Only Monospaced Font"), "Show Only Monospaced Font", "bool", "", tr("Show only monospaced fonts when choosing a font"), "Show only monospaced fonts when choosing a font", "", false, true, true, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, true, QVariant()});
    settings.append(SettingInfo {"Total Usage Time", "Total Usage Time", "Total Usage Time", "int", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, 0, QVariant()});
    settings.append(SettingInfo {"Promotion Dialog Shown", "Promotion Dialog Shown", "Promotion Dialog Shown", "bool", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Auto Uncheck Accepted Testcases", tr("Auto Uncheck Accepted Testcases"), "Auto Uncheck Accepted Testcases", "bool", "", tr("Automatically uncheck test cases when they get accepted."), "Automatically uncheck test cases when they get accepted.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Full Screen Dialog Shown", "Full Screen Dialog Shown", "Full Screen Dialog Shown", "bool", "", tr(""), "", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"WakaTime/Path", tr("Path"), "Path", "QString", "PathItem", tr("The path to the WakaTime executable file"), "The path to the WakaTime executable file", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"WakaTime/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, "wakatime", PathItem::Executable});
    settings.append(SettingInfo {"WakaTime/Api Key", tr("Api Key"), "Api Key", "QString", "", tr("Can be found at https://wakatime.com/settings/account.\nIt can be empty if you have the global wakatime config file ~/.wakatime.cfg."), "Can be found at https://wakatime.com/settings/account.\nIt can be empty if you have the global wakatime config file ~/.wakatime.cfg.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"WakaTime/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, "", QVariant()});
    settings.append(SettingInfo {"WakaTime/Enable", tr("Enable WakaTime"), "Enable WakaTime", "bool", "", tr("Use WakaTime to track your time usage. The WakaTime CLI needs to be installed."), "Use WakaTime to track your time usage. The WakaTime CLI needs to be installed.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"WakaTime/Proxy", tr("Use Proxy"), "Use Proxy", "bool", "", tr("Use Advanced->Network Proxy to send data to WakaTime."), "Use Advanced->Network Proxy to send data to WakaTime.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"WakaTime/Enable", [](const QVariant &var) { return var.toBool(); }}, }, {}, false, QVariant()});
    settings.append(SettingInfo {"Display Stopwatch", tr("Display Stopwatch"), "Display Stopwatch", "bool", "", tr("Show a stopwatch in the UI. You can use it to track your time spent on solving a problem."), "Show a stopwatch in the UI. You can use it to track your time spent on solving a problem.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, false, QVariant()});
    settings.append(SettingInfo {"Toggle Stopwatch On Tab Switch", tr("Start/Stop stopwatch on tab switch"), "Start/Stop stopwatch on tab switch", "bool", "", tr("When switching to a different tab, automatically start the stopwatch\non the current tab and pause the stopwatch on the previous tab."), "When switching to a different tab, automatically start the stopwatch\non the current tab and pause the stopwatch on the previous tab.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Display Stopwatch", [](const QVariant &var) { return var.toBool(); }}, }, {}, false, QVariant()});
    settings.append(SettingInfo {"Hide Stopwatch Result", tr("Show stopwatch result only when the button is pressed"), "Show stopwatch result only when the button is pressed", "bool", "", tr("Hide the time of the stopwatch and only show the time when the \"Show\" button is pressed.\nThis may reduce distractions caused by stopwatch updates."), "Hide the time of the stopwatch and only show the time when the \"Show\" button is pressed.\nThis may reduce distractions caused by stopwatch updates.", "", false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {{"Display Stopwatch", [](const QVariant &var) { return var.toBool(); }}, }, {}, false, QVariant()});
    settings.append(SettingInfo {"Error Message Color", tr("Color of error messages"), "Color of error messages", "QString", "QComboBox", tr("Color of error messages"), "Color of error messages", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "red", QStringList {"aqua", "black", "blue", "fuchsia", "gray", "green", "lime", "maroon", "navy", "olive", "orange", "purple", "red", "silver", "teal", "white", "yellow"}});
    settings.append(SettingInfo {"Warn Message Color", tr("Color of warning messages"), "Color of warning messages", "QString", "QComboBox", tr("Color of warning messages"), "Color of warning messages", "", true, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "green", QStringList {"aqua", "black", "blue", "fuchsia", "gray", "green", "lime", "maroon", "navy", "olive", "orange", "purple", "red", "silver", "teal", "white", "yellow"}});
    settings.append(SettingInfo {"Default Path/Action/Open File/Uses", tr("Default path used for %1").arg(tr("Open File")), "Default Path Action Open File Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Open File")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${file}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Open File/Changes", tr("Default paths changed by %1").arg(tr("Open File")), "Default Path Action Open File Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Open File")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "file, testcase, checker", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Save File/Uses", tr("Default path used for %1").arg(tr("Save File")), "Default Path Action Save File Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Save File")) + "\n" + tr("It can be overridden by %1.").arg(tr("Default File Paths For Problem URLs")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${file}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Save File/Changes", tr("Default paths changed by %1").arg(tr("Save File")), "Default Path Action Save File Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Save File")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "file, testcase, checker", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Open Contest/Uses", tr("Default path used for %1").arg(tr("Open Contest")), "Default Path Action Open Contest Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Open Contest")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${contest}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Open Contest/Changes", tr("Default paths changed by %1").arg(tr("Open Contest")), "Default Path Action Open Contest Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Open Contest")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "contest, file, testcase, checker", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Load Single Test Case/Uses", tr("Default path used for %1").arg(tr("Load Single Test Case")), "Default Path Action Load Single Test Case Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Load Single Test Case")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${testcase}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Load Single Test Case/Changes", tr("Default paths changed by %1").arg(tr("Load Single Test Case")), "Default Path Action Load Single Test Case Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Load Single Test Case")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "testcase", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Add Pairs Of Test Cases/Uses", tr("Default path used for %1").arg(tr("Add Pairs Of Test Cases")), "Default Path Action Add Pairs Of Test Cases Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Add Pairs Of Test Cases")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${testcase}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Add Pairs Of Test Cases/Changes", tr("Default paths changed by %1").arg(tr("Add Pairs Of Test Cases")), "Default Path Action Add Pairs Of Test Cases Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Add Pairs Of Test Cases")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "testcase", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Save Test Case To A File/Uses", tr("Default path used for %1").arg(tr("Save Test Case To A File")), "Default Path Action Save Test Case To A File Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Save Test Case To A File")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${testcase}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Save Test Case To A File/Changes", tr("Default paths changed by %1").arg(tr("Save Test Case To A File")), "Default Path Action Save Test Case To A File Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Save Test Case To A File")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "testcase", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Custom Checker/Uses", tr("Default path used for %1").arg(tr("Custom Checker")), "Default Path Action Custom Checker Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Custom Checker")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${checker}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Custom Checker/Changes", tr("Default paths changed by %1").arg(tr("Custom Checker")), "Default Path Action Custom Checker Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Custom Checker")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "checker", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Export And Import Settings/Uses", tr("Default path used for %1").arg(tr("Export And Import Settings")), "Default Path Action Export And Import Settings Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Export And Import Settings")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${settings}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Export And Import Settings/Changes", tr("Default paths changed by %1").arg(tr("Export And Import Settings")), "Default Path Action Export And Import Settings Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Export And Import Settings")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "settings", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Export And Load Session/Uses", tr("Default path used for %1").arg(tr("Export And Load Session")), "Default Path Action Export And Load Session Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Export And Load Session")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${session}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Export And Load Session/Changes", tr("Default paths changed by %1").arg(tr("Export And Load Session")), "Default Path Action Export And Load Session Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Export And Load Session")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "session", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Extract And Load Snippets/Uses", tr("Default path used for %1").arg(tr("Extract And Load Snippets")), "Default Path Action Extract And Load Snippets Uses", "QString", "", tr("The default path used when choosing a path for %1.\nYou can use ${<default path name>} as a place holder.").arg(tr("Extract And Load Snippets")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "${snippets}", QVariant()});
    settings.append(SettingInfo {"Default Path/Action/Extract And Load Snippets/Changes", tr("Default paths changed by %1").arg(tr("Extract And Load Snippets")), "Default Path Action Extract And Load Snippets Changes", "QString", "", tr("The default paths changed after choosing a path for %1.\nIt is a list of <default path name>s, separated by commas, and can be empty.").arg(tr("Extract And Load Snippets")), "", tr("default-paths", "the anchor of Default Paths on https://cpeditor.org/docs/preferences/file-path"), false, true, false, [](SettingInfo *info, ValueWidget *widget, QWidget *parent){  }, {}, {}, "snippets", QVariant()});
};
